/* Copyright (c) 1996 University of Utah.  All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for NON-COMMERCIAL purposes and without fee is
 * hereby granted provided that this copyright notice appears in all
 * copies.
 *
 * See http://www.cs.utah.edu/~hamlet for more educational resources.
 *
 * The University of Utah MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT
 * THE SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED,
 * INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, OR
 * NON-INFRINGEMENT. The University of Utah SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package Interest;

import java.awt.*;
import java.io.*;
import hamlet.*;

public class RateEntry extends Panel {

    ScrollableNumbers balance;
    ScrollableNumbers rate;
    ScrollableNumbers intervals;

    Checkbox simple;
    Checkbox compound;
    Checkbox continuous;

    Checkbox chosen = null;

    Panel balancePanel, ratePanel, intervalsPanel;


    public RateEntry (Color c) {

	setForeground(c);
	setLayout(new GridLayout(4,1));

	simple = new Checkbox("Simp");
	compound = new Checkbox("Comp");
	continuous = new Checkbox("Cont");
	
	Panel interest = new Panel();
	interest.setLayout(new FlowLayout(FlowLayout.RIGHT));
	interest.add(simple);
	interest.add(compound);
	interest.add(continuous);

	add(interest);
	
	balance = new ScrollableNumbers(100.,
					10.,
					100.,
					10.,
					1000.,
					4,
					ScrollableNumbers.LEFT,
					"Balance ",
					this);

	rate = new ScrollableNumbers(5.,
				     .1,
				     1.,
				     0.,
				     50.,
				     4,
				     ScrollableNumbers.LEFT,
				     "Rate ",
				     this);

	intervals = new ScrollableNumbers(1,
					  1,
					  10,
					  1,
					  365,
					  4,
					  ScrollableNumbers.LEFT,
					  "Intervals ",
					  this);

	balancePanel = new Panel();
	balancePanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	balancePanel.add(balance);
	add(balancePanel);

	ratePanel = new Panel();
	ratePanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	ratePanel.add(rate);
	add(ratePanel);

	intervalsPanel = new Panel();
	intervalsPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
	intervalsPanel.add(intervals);
	add(intervalsPanel);
	
	balancePanel.show(false);
	ratePanel.show(false);
	intervalsPanel.show(false);

    }
	

    public void setCurve (String type, double rte, double bal, int ints) {

        rate.setDoubleText(rte);
	balance.setDoubleText(bal);
	intervals.setIntText(ints);
	chosen = null;

	if (type.equals("simple")) {
	    postEvent(new Event(simple, Event.ACTION_EVENT, null));
	    simple.setState(true);
	}

	else if (type.equals("compound")) {
	    postEvent(new Event(compound, Event.ACTION_EVENT, null));
	    compound.setState(true);
	}

	else if (type.equals("continuous")) {
	    postEvent(new Event(continuous, Event.ACTION_EVENT, null));
	    continuous.setState(true);
	}

    }
    

    public void reset () {
	if (chosen != null) chosen.setState(false);
	chosen = null;
	rate.setDoubleText(5);
	balance.setDoubleText(100);
	intervals.setIntText(1);
	balancePanel.show(false);
	ratePanel.show(false);
	intervalsPanel.show(false);
    }
	    

    public boolean action (Event theEvent, Object arg) {

	if (theEvent.target instanceof Checkbox) {

	    if (theEvent.target == simple) {
		if (chosen == simple) {
		    simple.setState(false);
		    chosen = null;
		}
		else {
		    chosen = simple;
		    balancePanel.show(true);
		    ratePanel.show(true);
		    intervalsPanel.show(false);
		    compound.setState(false);
		    continuous.setState(false);
		}
	    }

	    else if (theEvent.target == compound) {
		if (chosen == compound) {
		    compound.setState(false);
		    chosen = null;
		}
		else {
		    chosen = compound;
		    balancePanel.show(true);
		    ratePanel.show(true);
		    intervalsPanel.show(true);
		    simple.setState(false);
		    continuous.setState(false);
		}
	    }

	    else if (theEvent.target == continuous) {
		if (chosen == continuous) {
		    chosen = null;
		    continuous.setState(false);
		}
		else {
		    chosen = continuous;
		    balancePanel.show(true);
		    ratePanel.show(true);
		    intervalsPanel.show(false);
		    simple.setState(false);
		    compound.setState(false);
		}
	    }

	    if (chosen == null) {
		balancePanel.show(false);
		ratePanel.show(false);
		intervalsPanel.show(false);
	    }

	    describe();
	    return(true);

	}

	else if (theEvent.target instanceof ScrollableNumbers) {
	    describe();
	    return(true);
	}

	return(false);

    }

    
    private void describe () {

	Invest investment;

	if (chosen == simple)
	    investment = new Invest(Invest.SIMPLE, 
				    balance.getDoubleText(),
				    rate.getDoubleText());

	else if (chosen == compound)
	    investment = new Invest(Invest.COMPOUND,
				    balance.getDoubleText(),
				    rate.getDoubleText(),
				    intervals.getIntText());

	else if (chosen == continuous)
	    investment = new Invest(Invest.CONTINUOUS,
				    balance.getDoubleText(),
				    rate.getDoubleText());
	
	else
	    investment = null;

	Event event = new Event(this, Event.ACTION_EVENT, investment);
	getParent().deliverEvent(event);

    }


    public Dimension preferredSize () {
	return(new Dimension(250,150));
    }
    
}
